import time
from flask import Blueprint, request
from oshepherd.api.utils import streamify_json
from oshepherd.api.generate.models import GenerateRequest

generate_blueprint = Blueprint("generate", __name__, url_prefix="/api/generate")


@generate_blueprint.route("/", methods=["POST"])
def generate():
    # TODO use .send_task() instead?
    from oshepherd.worker.tasks import make_generate_request

    print(f" # request.json {request.json}")
    generate_request = GenerateRequest(**request.json)

    # req as json string ready to be sent though broker
    generate_request_json_str = generate_request.model_dump_json()
    print(generate_request_json_str)

    # queue request to remote ollama api server though
    task = make_generate_request.delay(generate_request_json_str)
    while not task.ready():
        print(" > waiting for response...")
        time.sleep(1)
    ollama_res = task.get(timeout=1)

    status = 200
    if ollama_res.get("error"):
        ollama_res = {
            "error": "Internal Server Error",
            "message": f"error triggering llm inference: {ollama_res['error']['message']}",
        }
        status = 500

    print(f" $ ollama response {status}: {ollama_res}")

    return streamify_json(ollama_res, status)



from pydantic import BaseModel
from typing import Optional


class WorkerConfig(BaseModel):
    RABBITMQ_URL: str
    REDIS_URL: str
    LOGLEVEL: Optional[str] = "info"
    CONCURRENCY: Optional[int] = 1
    PREFETCH_MULTIPLIER: Optional[int] = 1
    RESULTS_EXPIRES: Optional[int] = 3600



import json
import ollama
from oshepherd.worker.app import celery_app


@celery_app.task(name="oshepherd.worker.tasks.make_generate_request")
def make_generate_request(request_str: str):
    try:
        request = json.loads(request_str)
        print(f"# make_generate_request request {request}")

        response = ollama.generate(**request)
        print(f"  $ success {response}")
    except Exception as error:
        print(f"  * error make_generate_request {error}")
        response = {
            "error": {"type": str(error.__class__.__name__), "message": str(error)}
        }
        print(
            f"    * error response {response}",
        )

    return response



from setuptools import setup, find_packages

setup(
    name="oshepherd",
    version="0.0.4",
    description="The Oshepherd guiding the Ollama(s) inference orchestration.",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author="mnemonica.ai",
    author_email="info@mnemonica.ai",
    packages=find_packages(),
    python_requires=">=3.8",
    install_requires=[
        "flask",
        "celery",
        "click",
        "ollama",
        "amqp",
        "redis",
        "pydantic",
        "python-dotenv",
        "requests",
    ],
    extras_require={
        'dev': [
            "packageName[tests, lint]",
            "build",
            "twine"
        ],
        'tests': [
            "pytest"
        ],
        'lint': [
            "black"
        ]
    },
    entry_points={
        "console_scripts": [
            "oshepherd=oshepherd.cli:main",
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    project_urls={
        "Homepage": "https://github.com/mnemonica-ai/oshepherd",
        "Issues": "https://github.com/mnemonica-ai/oshepherd/issues",
    },
)
